--[[
Author: Starinnia
CPR is a combo points display addon based on Funkydude's BasicComboPoints
$Date: 2010-07-20 20:03:56 +0000 (Tue, 20 Jul 2010) $
$Revision: 95 $
Project Version: r97
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2007-2010 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

local ComboPointsRedux = LibStub("AceAddon-3.0"):NewAddon("ComboPointsRedux", "AceConsole-3.0")
ComboPointsRedux:SetDefaultModuleState(false)
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ComboPointsRedux")

local basepath = [[Interface\Addons\ComboPointsRedux\Icons\]]

function ComboPointsRedux:OnInitialize()
	local defaults = {
		profile = {
			modules = {
				['*'] = {
					enabled = true,
					--text options
					font = "Friz Quadrata TT",
					fontsize = 12,
					textColor = {1, 1, 1},
					textAlpha = 1,
					textStrata = "HIGH",
					textPos = {},
					disableText = false,
					--graphics options
					orientation = "h",
					icon = "square",
					oneColor = {1, 0.9, 0},
					twoColor = {1, 0.9, 0},
					threeColor = {1, 0.9, 0},
					fourColor = {1, 0.9, 0},
					fiveColor = {1, 0, 0},
					graphicsAlpha = 1,
					spacing = 5,
					scale = 1,
					strata = "HIGH",
					graphicsPos = {},
					disableGraphics = false,
				},
				['Deadly Poison'] = {
					enabled = true,
					--text options
					font = "Friz Quadrata TT",
					fontsize = 12,
					textColor = {1, 1, 1},
					textAlpha = 1,
					textStrata = "HIGH",
					textPos = {},
					disableText = false,
					--graphics options
					orientation = "h",
					icon = "square",
					oneColor = {0, 1, 0},
					twoColor = {0, 1, 0},
					threeColor = {0, 1, 0},
					fourColor = {0, 1, 0},
					fiveColor = {0, 0.5, 0},
					graphicsAlpha = 1,
					spacing = 5,
					scale = 1,
					strata = "HIGH",
					graphicsPos = {},
					disableGraphics = false,
				}
			},
			locked = false,
		}
	}
	
	self.db = LibStub("AceDB-3.0"):New("CPRDB2", defaults, "Default")
	
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
	self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
	
	self:RegisterChatCommand("cpr", "OpenConfig", true, true)
	self:RegisterChatCommand("cpredux", "OpenConfig", true, true)
end

function ComboPointsRedux:OnEnable()
	for name, module in self:IterateModules() do
		if self.db.profile.modules[name].enabled then
			module:Enable()
		end
	end
end

function ComboPointsRedux:Reset()
	for name, module in self:IterateModules() do
		if module:IsEnabled() then module:ResetSettings() end
	end
end

function ComboPointsRedux:Refresh()
	for name, module in self:IterateModules() do
		--pass true to :UpdateSettings forces the frame anchors to update as well
		if module:IsEnabled() then module:UpdateSettings(true) end
	end
end

function ComboPointsRedux:OpenConfig()
	LoadAddOn("ComboPointsRedux_Options")
	LibStub("AceConfigDialog-3.0"):Open("ComboPointsRedux")
end

do
	local pointsToColors = {[0] = "oneColor", [1] = "oneColor", [2] = "twoColor", [3] = "threeColor", [4] = "fourColor", [5] = "fiveColor"}
	function ComboPointsRedux:GetColorByPoints(module, points)
		local key = pointsToColors[points]
		return unpack(self.db.profile.modules[module][key])
	end
end

local bg = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",}
function ComboPointsRedux:MakeTextFrame(module)
	local f = CreateFrame("FRAME", nil, UIParent)
	local db = self.db.profile.modules[module]
	f:SetBackdrop(bg)
	f:SetFrameStrata(db.textStrata)
	f:SetClampedToScreen(true)
	if db.textPos.x then
		f:SetPoint(db.textPos.point, UIParent, db.textPos.anchor, db.textPos.x, db.textPos.y)
	else
		f:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end
	f:SetBackdropColor(1,1,1,0)
	f:SetWidth(50)
	f:SetHeight(50)
	
	f.count = f:CreateFontString(nil,"OVERLAY")
	f.count:SetAllPoints(f)
	f.count:SetShadowColor(0, 0, 0, db.textAlpha)
	f.count:SetShadowOffset(1, -1)
	f.count:SetFont(LSM:Fetch("font", db.font), db.fontsize)
	f.count:SetTextColor(unpack(db.textColor))
	f.count:SetAlpha(db.textAlpha)
	
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", function(frame) frame:StartMoving() end)
	f:SetScript("OnDragStop", function(frame)
		frame:StopMovingOrSizing()
		local point, _, anchor, x, y = frame:GetPoint()
		db.textPos.x = math.floor(x)
		db.textPos.y = math.floor(y)
		db.textPos.anchor = anchor
		db.textPos.point = point
	end)
	
	if not self.db.profile.locked then
		f:SetBackdropColor(1,1,1,1)
		f:EnableMouse(true)
		f:SetMovable(true)
		f.count:SetText(self.modules[module].abbrev)
	else
		f:EnableMouse(false)
		f:SetMovable(false)
	end
	
	function f:SetNumPoints(points)
		self.count:SetText(points)
	end
	
	if db.disableText then
		f:Hide()
	end
	
	return f
end

function ComboPointsRedux:MakeGraphicsFrame(module)
	local g = CreateFrame("FRAME", nil, UIParent)
	local db = self.db.profile.modules[module]
	g:SetBackdrop(bg)
	g:SetFrameStrata(db.strata)
	g:SetClampedToScreen(true)
	if db.graphicsPos.x then
		g:SetPoint(db.graphicsPos.point, UIParent, db.graphicsPos.anchor, db.graphicsPos.x, db.graphicsPos.y)
	else
		g:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end
	
	g:SetBackdropColor(1,1,1,0)
	
	if db.orientation == "v" then
		g:SetHeight(145*db.scale)
		g:SetWidth(25*db.scale)
	else
		g:SetWidth(145*db.scale)
		g:SetHeight(25*db.scale)
	end
	
	g:RegisterForDrag("LeftButton")
	g:SetScript("OnDragStart", function(frame) frame:StartMoving() end)
	g:SetScript("OnDragStop", function(frame)
		frame:StopMovingOrSizing()
		local point, _, anchor, x, y = frame:GetPoint()
		db.graphicsPos.x = math.floor(x)
		db.graphicsPos.y = math.floor(y)
		db.graphicsPos.anchor = anchor
		db.graphicsPos.point = point
	end)
	
	g.points = {}
	for i = 1, 5 do
		g.points[i] = CreateFrame("FRAME", nil, g)
		g.points[i].icon = g.points[i]:CreateTexture(nil, "OVERLAY")
		g.points[i].icon:SetAllPoints(g.points[i])
		g.points[i].icon:SetTexture(basepath..db.icon)
		g.points[i].icon:SetVertexColor(unpack(db.oneColor))
		g.points[i]:SetAlpha(db.graphicsAlpha)
		g.points[i]:SetHeight(25*db.scale)
		g.points[i]:SetWidth(25*db.scale)
		g.points[i]:Hide()
	end
	
	if not self.db.profile.locked then
		g:SetBackdropColor(1,1,1,1)
		g:EnableMouse(true)
		g:SetMovable(true)
		g.points[1]:Show()
		g.points[2]:Show()
	else
		g:EnableMouse(false)
		g:SetMovable(false)
	end
	
	local offset = db.spacing*db.scale
	if db.orientation == "v" then
		g.points[1]:SetPoint("BOTTOM", g, "BOTTOM", 0, 0)
		g.points[2]:SetPoint("BOTTOM", g.points[1], "TOP", 0, offset)
		g.points[3]:SetPoint("BOTTOM", g.points[2], "TOP", 0, offset)
		g.points[4]:SetPoint("BOTTOM", g.points[3], "TOP", 0, offset)
		g.points[5]:SetPoint("BOTTOM", g.points[4], "TOP", 0, offset)
	else
		g.points[1]:SetPoint("LEFT", g, "LEFT", 0, 0)
		g.points[2]:SetPoint("LEFT", g.points[1], "RIGHT", offset, 0)
		g.points[3]:SetPoint("LEFT", g.points[2], "RIGHT", offset, 0)
		g.points[4]:SetPoint("LEFT", g.points[3], "RIGHT", offset, 0)
		g.points[5]:SetPoint("LEFT", g.points[4], "RIGHT", offset, 0)
	end
	
	if db.disableGraphics then
		f:Hide()
	end
	
	return g
end
